`filter_peaks_in_spectrum` <-
function(mz_values, intensities, peak_indexes, offset="mean", 
                                     deviation="sd", scale="local", smooth=TRUE, quantile=NULL, 
                                     sn_ratio=5, min_mz=min(mz_values, na.rm=TRUE), max_mz=max(mz_values, na.rm=TRUE), perc_width=0.3) {
  # Correct for background noise.
  if (offset != "none" | deviation != "none" | smooth) {
    #print(paste("Using background noise estimation method:", offset, deviation))

    noisedev <- calc_offset_dev(intensities, offset_method=offset, 
                               dev_method=deviation, scale=scale)

    if (smooth == TRUE) {
      if (offset == "none") { noisedev$offset <- intensities; }
      noisedev$offset    <- supsmu(mz_values, noisedev$offset)$y
      noisedev$deviation <- supsmu(mz_values, noisedev$deviation)$y
    }

    noise_peaks <- which( intensities[peak_indexes] < 
                           noisedev$offset[peak_indexes] + 
                           (sn_ratio * noisedev$dev[peak_indexes]) )

    print(paste(length(noise_peaks), "noise peaks identified out of a total of", 
                length(peak_indexes), "peaks"))
    if (length(noise_peaks) > 0) {
      peak_indexes <- peak_indexes[-noise_peaks]
    }
  }

  if (min_mz > min(mz_values, na.rm=TRUE) | max_mz < max(mz_values, na.rm=TRUE)) {
    to_remove   <- which(mz_values[peak_indexes] < min_mz | mz_values[peak_indexes] > max_mz);
    print(paste(length(to_remove), "peaks identified that are outside the region between", 
                                   min_mz, "and", max_mz, "m/z"))
    if (length(to_remove) > 0) {
      peak_indexes <- peak_indexes[-to_remove]
    }
  }

  if (!is.null(quantile)) {
    # Discard values below quantile
    nonsig <- which(intensities[peak_indexes] < 
                    quantile(intensities, prob=seq(0, 1, 0.001))[quantile*10])
    print(paste(length(nonsig), "peaks identified that are below quantile", quantile))
    if (length(nonsig) > 0) {
      peak_indexes <- peak_indexes[-nonsig]
    }
  }

  if (perc_width > 0) {
    # Remove non-maximal local peaks
    max_peak_width   <- log10(1 + (perc_width / 100));
    prox_hits <- which(diff(log10(mz_values[peak_indexes])) < (max_peak_width/2))
    outtakes <- c();
    if (length(prox_hits) > 1) {
      for (i in prox_hits[-1]) {
        # This is mainly for cases where values are Inf.
        # This is usually due to bad normalisation.
        if (intensities[peak_indexes][i-1] == intensities[peak_indexes][i]) {
          outtakes <- c(outtakes, i-1);
        } else {
          if (diff(intensities[peak_indexes][c(i-1,i)]) > 0) {
            outtakes <- c(outtakes, i-1);
          } else {
            outtakes <- c(outtakes, i);
          }
        }
      }
    }
    print(paste(length(unique(outtakes)), "peaks identified that are non-maximal local"))
    if (length(outtakes) > 0) {
      peak_indexes <- peak_indexes[-unique(outtakes)];
    }
  }

  print(paste("Resulting number of peaks:", length(peak_indexes)))

  return(peak_indexes);
}

